/* 
  ========================================
  EEPROM write/read functions
  ========================================
  
*/
/*

This code started with Chris Church's OpenMoco Time Lapse Engine and was modified by Brian Burling
to work with the eMotimo PT.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.



*/




boolean eeprom_saved() {
  
    // read eeprom saved status
    
  byte saved = EEPROM.read(0);
 
   // EEPROM memory is by default set to 1, so we
   // set it to zero if we've written data to eeprom
  return( ! saved );
}

void eeprom_saved( boolean saved ) {
  // set eeprom saved status
  
   // EEPROM memory is by default set to 1, so we
   // set it to zero if we've written data to eeprom
  
  EEPROM.write(0, !saved);
}





// One can ask why I didn't use the templates from http://www.arduino.cc/playground/Code/EEPROMWriteAnything
// The primary reason here is that we're going to be calling these functions OFTEN, and I _really_ don't 
// want the templates getting inlined _everywhere_, what a mess!  So, rather than be slick, let's just declare
// what we mean, and do it once - forget about the overhead of the function call, and worry more about
// flash and stack abuse 


void eeprom_write( int pos, byte& val, byte len ) {
  byte* p = (byte*)(void*)&val;
  for( byte i = 0; i < len; i++ )
    EEPROM.write(pos++, *p++);    

    // indicate that memory has been saved
  eeprom_saved(true);

}
    
void eeprom_write( int pos, unsigned int& val ) {
  byte* p = (byte*)(void*)&val;   
  eeprom_write(pos, *p, sizeof(int));  
}

void eeprom_write( int pos, unsigned long& val ) {
  byte* p = (byte*)(void*)&val;   
  eeprom_write(pos, *p, sizeof(long));    
}

void eeprom_write( int pos, float& val ) {
  byte* p = (byte*)(void*)&val;   
  eeprom_write(pos, *p, sizeof(float));    
}

void eeprom_write( int pos, byte& val ) {  
  EEPROM.write(pos, val);
    // indicate that memory has been saved
  eeprom_saved(true);
}





 // read functions

void eeprom_read( int pos, byte& val, byte len ) {
  byte* p = (byte*)(void*)&val;
  for(byte i = 0; i < len; i++) 
    *p++ = EEPROM.read(pos++);
}

void eeprom_read( int pos, byte& val ) {
  val = EEPROM.read(pos);
}


void eeprom_read( int pos, int& val ) {
  byte* p = (byte*)(void*)&val;
  eeprom_read(pos, *p, sizeof(int));
}

void eeprom_read( int pos, unsigned int& val ) {

  byte* p = (byte*)(void*)&val;
  eeprom_read(pos, *p, sizeof(int));
  
}

void eeprom_read( int pos, unsigned long& val ) {

  byte* p = (byte*)(void*)&val;
  eeprom_read(pos, *p, sizeof(long));
  
}

void eeprom_read( int pos, float& val ) {

  byte* p = (byte*)(void*)&val;
  eeprom_read(pos, *p, sizeof(float));
  
}
    

void set_defaults_in_ram() {

first_time=1;//			Serial.println(first_time);
intval=20;//			Serial.println(intval);
interval=2000;//			Serial.println(interval);
camera_exp_tm=100;//			Serial.println(camera_exp_tm);
camera_fired=0;//			Serial.println(camera_fired);
camera_moving_shots=200;//			Serial.println(camera_moving_shots);
camera_total_shots=0;//			Serial.println(camera_total_shots);
overaldur=30;//			Serial.println(overaldur);
prefire_time=1;//			Serial.println(prefire_time);
rampval=50;//			Serial.println(rampval);
static_tm=1;//			Serial.println(static_tm);
lead_in=1;//			Serial.println(lead_in);
lead_out=1;//			Serial.println(lead_out);
motor_steps_end[0]=0.0;//			Serial.println(motor_steps_end[0]);
motor_steps_end[1]=0.0;//			Serial.println(motor_steps_end[1]);
motor_steps_end[2]=0.0;//			Serial.println(motor_steps_end[2]);
keyframe[0]=0;//			Serial.println(keyframe[0]);
keyframe[1]=0;//			Serial.println(keyframe[1]);
keyframe[2]=0;//			Serial.println(keyframe[2]);
keyframe[3]=0;//			Serial.println(keyframe[3]);
keyframe[4]=0;//			Serial.println(keyframe[4]);
keyframe[5]=0;//			Serial.println(keyframe[5]);
linear_steps_per_shot[0]=0.0;//			Serial.println(linear_steps_per_shot[0]);
linear_steps_per_shot[1]=0.0;//			Serial.println(linear_steps_per_shot[1]);
linear_steps_per_shot[2]=0.0;//			Serial.println(linear_steps_per_shot[2]);
ramp_params_steps[0]=0.0;//			Serial.println(ramp_params_steps[0]);
ramp_params_steps[1]=0.0;//			Serial.println(ramp_params_steps[1]);
ramp_params_steps[2]=0.0;//			Serial.println(ramp_params_steps[2]);
current_steps.x=0.0;//			Serial.println(current_steps.x);
current_steps.y=0.0;//			Serial.println(current_steps.y);
current_steps.z=0.0;//			Serial.println(current_steps.z);
progstep=0;//			Serial.println(progstep);
Program_Engaged=0;//			Serial.println(Program_Engaged);

}
void write_defaults_to_eeprom_memory() {
  
set_defaults_in_ram();
write_all_ram_to_eeprom();
}


void write_all_ram_to_eeprom()  {
//eeprom_write(1, first_time);	
eeprom_write(3, intval);	
eeprom_write(5, interval);	
eeprom_write(9, camera_exp_tm);	
eeprom_write(13, camera_fired);	
eeprom_write(15, camera_moving_shots);	
eeprom_write(17, camera_total_shots);	
eeprom_write(19, overaldur);	
eeprom_write(21, prefire_time);	
eeprom_write(23, rampval);	
eeprom_write(25, static_tm);	
eeprom_write(27, lead_in);	
eeprom_write(29, lead_out);	
eeprom_write(31, motor_steps_end[0]);	
eeprom_write(35, motor_steps_end[1]);	
eeprom_write(39, motor_steps_end[2]);	
eeprom_write(43, keyframe[0]);	
eeprom_write(45, keyframe[1]);	
eeprom_write(47, keyframe[2]);	
eeprom_write(49, keyframe[3]);	
eeprom_write(51, keyframe[4]);	
eeprom_write(53, keyframe[5]);	
eeprom_write(55, linear_steps_per_shot[0]);	
eeprom_write(59, linear_steps_per_shot[1]);	
eeprom_write(63, linear_steps_per_shot[2]);	
eeprom_write(67, ramp_params_steps[0]);	
eeprom_write(71, ramp_params_steps[1]);	
eeprom_write(75, ramp_params_steps[2]);	
eeprom_write(79, current_steps.x);	
eeprom_write(83, current_steps.y);	
eeprom_write(87, current_steps.z);	
eeprom_write(91, progstep);	
eeprom_write(93, Program_Engaged);	
}









 // restore memory
 
void restore_from_eeprom_memory() {

//eeprom_read(1, first_time);
eeprom_read(3, intval);
eeprom_read(5, interval);
eeprom_read(9, camera_exp_tm);
eeprom_read(13, camera_fired);
eeprom_read(15, camera_moving_shots);
eeprom_read(17, camera_total_shots);
eeprom_read(19, overaldur);
eeprom_read(21, prefire_time);
eeprom_read(23, rampval);
eeprom_read(25, static_tm);
eeprom_read(27, lead_in);
eeprom_read(29, lead_out);
eeprom_read(31, motor_steps_end[0]);
eeprom_read(35, motor_steps_end[1]);
eeprom_read(39, motor_steps_end[2]);
eeprom_read(43, keyframe[0]);
eeprom_read(45, keyframe[1]);
eeprom_read(47, keyframe[2]);
eeprom_read(49, keyframe[3]);
eeprom_read(51, keyframe[4]);
eeprom_read(53, keyframe[5]);
eeprom_read(55, linear_steps_per_shot[0]);
eeprom_read(59, linear_steps_per_shot[1]);
eeprom_read(63, linear_steps_per_shot[2]);
eeprom_read(67, ramp_params_steps[0]);
eeprom_read(71, ramp_params_steps[1]);
eeprom_read(75, ramp_params_steps[2]);
eeprom_read(79, current_steps.x);
eeprom_read(83, current_steps.y);
eeprom_read(87, current_steps.z);
//eeprom_read(91, progstep);
eeprom_read(93, Program_Engaged);
  
  
//delay(100);
}

void review_RAM_Contents() {

//Serial.println(flash_enabled);
Serial.println(first_time);
Serial.println(intval);
Serial.println(interval);
Serial.println(camera_exp_tm);
Serial.println(camera_fired);
Serial.println(camera_moving_shots);
Serial.println(camera_total_shots);
Serial.println(overaldur);
Serial.println(prefire_time);
Serial.println(rampval);
Serial.println(static_tm);
Serial.println(lead_in);
Serial.println(lead_out);
Serial.println(motor_steps_end[0]);
Serial.println(motor_steps_end[1]);
Serial.println(motor_steps_end[2]);
Serial.println(keyframe[0]);
Serial.println(keyframe[1]);
Serial.println(keyframe[2]);
Serial.println(keyframe[3]);
Serial.println(keyframe[4]);
Serial.println(keyframe[5]);
Serial.println(linear_steps_per_shot[0]);
Serial.println(linear_steps_per_shot[1]);
Serial.println(linear_steps_per_shot[2]);
Serial.println(ramp_params_steps[0]);
Serial.println(ramp_params_steps[1]);
Serial.println(ramp_params_steps[2]);
Serial.println(current_steps.x);
Serial.println(current_steps.y);
Serial.println(current_steps.z);
Serial.println(progstep);
Serial.println(Program_Engaged);




}




