/*

This code started with OpenMoco Time Lapse Engine openmoco.org and was modified by Brian Burling
to work with eMotimo product.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

//issue with joystick3 in negative direction
//backlight handling 
//run through reinit with joystick test - it presses buttons on restart - don't allow this.

/* 

 =========================================
Main Program
 =========================================
 
*/

//Setup for Nunchuck
#include <Wire.h>
#include <MsTimer2.h>
#include <SoftwareSerial.h> 
#include <EEPROM.h>
#include <WiiNunchuck2.h>

#include <NHDLCD7.h>  //BB for LCD
NHDLCD7 lcd(4,2,16); // desired pin, rows, cols   //BB for LCD

char lcdbuffer1[16];

prog_char setup_0[] PROGMEM = "Setup Complete";   // "String 0" etc are strings to store - change to suit.
prog_char setup_1[] PROGMEM = "  TB3 1_6_3_35  ";
prog_char setup_2[] PROGMEM = "Connect Joystick";
prog_char setup_3[] PROGMEM = "C-Next Z-Go Back";
prog_char setup_4[] PROGMEM = "C-Use Stored";
prog_char setup_5[] PROGMEM = "Z-Reset";
prog_char setup_6[] PROGMEM = "Params Reset";
prog_char setup_7[] PROGMEM = "Low Power"; //free
prog_char setup_8[] PROGMEM = "Move to Start Pt";
prog_char setup_9[] PROGMEM = "Start Pt. Set";
prog_char setup_10[] PROGMEM = " "; //free
prog_char setup_11[] PROGMEM = "Moving to stored";
prog_char setup_12[] PROGMEM = "end point";
prog_char setup_13[] PROGMEM = "Confirm or Move";
prog_char setup_14[] PROGMEM = "C-Next";
prog_char setup_15[] PROGMEM = "Move to End Pt.";
prog_char setup_16[] PROGMEM = "End Point Set";
prog_char setup_17[] PROGMEM = "Set Sht Interval";
prog_char setup_18[] PROGMEM = "Intval:   .  sec";
prog_char setup_19[] PROGMEM = " Ext.Trig";
prog_char setup_20[] PROGMEM = " Video   ";
prog_char setup_21[] PROGMEM = "Interval Set";
prog_char setup_22[] PROGMEM = "Set Static Time";
prog_char setup_23[] PROGMEM = "Stat_T:   .  sec";
prog_char setup_24[] PROGMEM = " Video   ";
prog_char setup_25[] PROGMEM = "Static Time Set";
prog_char setup_26[] PROGMEM = "Set PreFire Time";
prog_char setup_27[] PROGMEM = " Pre_T:   .  sec";
prog_char setup_28[] PROGMEM = "Prefire Time Set";
prog_char setup_29[] PROGMEM = "    Set Ramp";
prog_char setup_30[] PROGMEM = "Ramp:     Frames";
prog_char setup_31[] PROGMEM = "Ramp Set";
prog_char setup_32[] PROGMEM = "Set Move";
prog_char setup_33[] PROGMEM = "Duration";
prog_char setup_34[] PROGMEM = "H:MM:SS";
prog_char setup_35[] PROGMEM = "Duration Set";
prog_char setup_36[] PROGMEM = "Set Static Lead";
prog_char setup_37[] PROGMEM = "In/Out Frames";
prog_char setup_38[] PROGMEM = "In      Out";
prog_char setup_39[] PROGMEM = "Lead Frames Set";
prog_char setup_40[] PROGMEM = " Going to Start";
prog_char setup_41[] PROGMEM = "Review and";
prog_char setup_42[] PROGMEM = "Confirm Setting";
prog_char setup_43[] PROGMEM = "Pan Steps:";
prog_char setup_44[] PROGMEM = "Tilt Steps:";
prog_char setup_45[] PROGMEM = "Cam Shots:";
prog_char setup_46[] PROGMEM = "Time:";
prog_char setup_47[] PROGMEM = "Ready?";
prog_char setup_48[] PROGMEM = "Press C Button";
prog_char setup_49[] PROGMEM = "Program Running";
prog_char setup_50[] PROGMEM = "Waiting for Ext.";
prog_char setup_51[] PROGMEM = "LeadIn";
prog_char setup_52[] PROGMEM = "RampUp";
prog_char setup_53[] PROGMEM = "Linear";
prog_char setup_54[] PROGMEM = "RampDn";
prog_char setup_55[] PROGMEM = "LeadOT";
prog_char setup_56[] PROGMEM = "Finish";
prog_char setup_57[] PROGMEM = "Center Joystick";
prog_char setup_58[] PROGMEM = "Program Complete";
prog_char setup_59[] PROGMEM = " Repeat Press C";
prog_char setup_60[] PROGMEM = "Battery too low";
prog_char setup_61[] PROGMEM = "  to continue";
prog_char setup_62[] PROGMEM = " Pause";
prog_char setup_63[] PROGMEM = "Enter Aux End Pt";
prog_char setup_64[] PROGMEM = "Aux End Pt. Set";


PROGMEM const char *setup_str[] = {setup_0,setup_1,setup_2,setup_3,setup_4,setup_5,setup_6,setup_7,setup_8,setup_9,setup_10,
setup_11,setup_12,setup_13,setup_14,setup_15,setup_16,setup_17,setup_18,setup_19,setup_20,
setup_21,setup_22,setup_23,setup_24,setup_25,setup_26,setup_27,setup_28,setup_29,setup_30,
setup_31,setup_32,setup_33,setup_34,setup_35,setup_36,setup_37,setup_38,setup_39,setup_40,
setup_41,setup_42,setup_43,setup_44,setup_45,setup_46,setup_47,setup_48,setup_49,setup_50,
setup_51,setup_52,setup_53,setup_54,setup_55,setup_56,setup_57,setup_58,setup_59,setup_60,
setup_61,setup_62,setup_63,setup_64};


//Global Parameters 
#define DEBUG 0 //
#define DEBUG_MOTOR 0 //
#define POWERSAVER true //set this to true to try minimizing power consumption during the loop by disabling motors between moves
#define POWERDOWN_LV true //set this to cause the TB3 to power down below 10 volts

//TB3 section  comment this out for PT code
///*
#define MOTORS 3

#define MOTOR0_STEP  5 
#define MOTOR1_STEP  6 
#define MOTOR2_STEP  7
#define MOTOR0_DIR   8 
#define MOTOR1_DIR   9 
#define MOTOR2_DIR   10

#define MOTOR_EN  17
#define MOTOR_EN2  11
#define MS1 15 
#define MS2 16 
#define MS3 16

#define IO_2  2 // drives middle of 2.5 mm connector on I/O port
#define IO_3  3 // drives tip of 2.5 mm connector on I/O port

#define CAMERA_PIN  12 // drives tip of 2.5 mm connector
#define FOCUS_PIN   13 // drives  middle of 2.5mm connector
//*/
//end TB3 section

//Start PT secions  
/*
#define MOTORS 3 //will clean this up later

#define MOTOR0_STEP  5
#define MOTOR1_STEP  6
#define MOTOR2_STEP  99 //not enabled for PT
#define MOTOR0_DIR   8
#define MOTOR1_DIR   9
#define MOTOR2_DIR   99 //not enabled for PT

#define MOTOR_EN     4
#define MOTOR_EN2    99 //not enabled for PT
#define MS1          3
#define MS2          7
#define MS3          10

#define IO_2  99 // not enabled for PT
#define IO_3  99 // not enabled for PT

#define CAMERA_PIN  12 // drives tip of 2.5 mm connector
#define FOCUS_PIN   13 // drives  middle of 2.5 connector
*/
//end PT section
// necessary control variables - store these in EEPROM

unsigned int  intval=20; //seconds x10  - used for the interval prompt and display
unsigned long interval = 2000; //calculated and is in ms
unsigned long camera_exp_tm    = 100; //length of shutter signal sent to camera
unsigned int  camera_fired     = 0; //number of shots fired
unsigned int  camera_moving_shots = 200; //frames for new duration/frames prompt
unsigned int  camera_total_shots= 0; //used at the end target for camera fired to compare against
unsigned int  overaldur=20; //seconds now for video only
unsigned int  prefire_time = 1; //currently hardcoded here to .2 second - prefire function not enabled.  this powerers up motor early for the shot
unsigned int  rampval=50;
unsigned int  static_tm=1; //new variable
unsigned int  lead_in=1;
unsigned int  lead_out=1;
unsigned int  start_delay_min=0;

//new motor routine values
float motor_steps_end[3]   = {0.0,0.0,0.0}; //end point target
unsigned int keyframe[6]= {0,0,0,0,0,0}; //this is basically the keyframes {start, end of rampup, start or rampdown, end}   - doesn't vary by motor at this point
float linear_steps_per_shot [MOTORS] = {0.0,0.0,0.0}; //{This is for the calculated or estimated steps per shot in a segment for each motor
float ramp_params_steps [MOTORS] = {0.0,0.0,0.0}; //This is to calc the steps at the end of rampup for each motor.  Each array value is for a motor

//Program Status Flags
boolean Program_Engaged=false;
boolean Shot_Sequence_Engaged=false;
boolean Prefire_Engaged=false;
boolean Shutter_Signal_Engaged=false;
boolean Static_Time_Engaged=false;
boolean IO_Engaged=false;
boolean Move_Engaged=false;
boolean Interrupt_Fire_Engaged=false;

//control varable, no need to store in EEPROM
unsigned int progstep = 0; //used to define case for main loop
boolean reset_prog=1; //used to handle program reset or used stored
unsigned int first_time=1; //variable to help with LCD dispay variable that need to show one time
int batt_low_cnt=0;
int max_shutter;
int max_prefire;
unsigned int program_progress=1;  //Lead in, ramp, linear, etc for motor routine case statement
unsigned long interval_tm        = 0;  //uc time to help with interval comparison
unsigned long interval_tm_last =0; //uc time to help with interval comparison
int cursorpos=1; //use 1 for left, 2 for right  - used for lead in, lead out
unsigned int lcd_dim_tm     = 10;
unsigned long input_last_tm = 0;
unsigned long diplay_last_tm = 0;
unsigned int  lcd_backlight_cur=80;
unsigned int  prompt_time=700; // in ms for delays of instructions
int  prompt_delay = 0; //to help with joystick reads and delays for inputs - this value is set during joystick read and executed later in the loop
unsigned int  video_sample_ms=50;
int reviewprog = 1;
//variables for display of remaining time
int timeh; 
int timem;
int time_s;
int aux_dist;

int NCReadStatus=0; //control variable for NC error handling

//Stepper Setup
//our feedrate variables.
//float feedrate = 0.0;
long  feedrate_micros = 0;

struct FloatPoint {
	float x;
	float y;
 	float z;
};

FloatPoint current_steps;
FloatPoint target_steps;
FloatPoint delta_steps;

//our direction vars
byte x_direction = 1;
byte y_direction = 1;
byte z_direction = 1;

//End setup of Steppers

/* 
 =========================================
 Setup functions
 =========================================
*/




void setup() {
 
  // setup motor pins
 pinMode(MOTOR0_STEP, OUTPUT);
 pinMode(MOTOR0_DIR, OUTPUT);
 pinMode(MOTOR1_STEP, OUTPUT);
 pinMode(MOTOR1_DIR, OUTPUT);
 pinMode(MOTOR2_STEP, OUTPUT);
 pinMode(MOTOR2_DIR, OUTPUT);
 
pinMode(MS1,OUTPUT);
pinMode(MS2,OUTPUT);
pinMode(MS3,OUTPUT);
 
digitalWrite(MS1, HIGH);
digitalWrite(MS2, HIGH);
digitalWrite(MS3, HIGH);

 pinMode(MOTOR_EN, OUTPUT);
 pinMode(MOTOR_EN2, OUTPUT);
 digitalWrite(MOTOR_EN, HIGH); //LOW Enables output, High Disables
 digitalWrite(MOTOR_EN2, HIGH); //LOW Enables output, High Disables
 
// setup camera pins
 pinMode(CAMERA_PIN, OUTPUT); 
 pinMode(FOCUS_PIN, OUTPUT); 
 
digitalWrite(CAMERA_PIN, LOW);
digitalWrite(FOCUS_PIN, LOW);

//Setup of I/0 Pings Start with output of I/Oport
pinMode(IO_2, OUTPUT);
pinMode(IO_3, OUTPUT);

digitalWrite(IO_2, LOW);
digitalWrite(IO_3, LOW);

pinMode(A0, INPUT); //this is for the voltage reading

//Setup LCD
 lcd.setup();  
 delay(100);
 draw(setup_str,0,1,2); // Setup Complete
 draw(setup_str,1,2,1); // Version Number
 lcd.contrast(50);
 lcd.cursorOff();
 
 delay(1000);
 lcd.empty();
 delay(100);

 draw(setup_str,2,1,1); // Connect Joystick
 lcd.bright(70);
 lcd.bright(80);
 input_last_tm= millis();

//Setup Serial Connection
//Serial.begin(57600);

//if (DEBUG) { 
  Serial.begin(115200);
  Serial.println("Starting Debug");
//}
 //begin  Setup for Nunchuck
Nunchuck.init(0);
delay(50);
for (int reads=1; reads<17; reads++) {
   Nunchuck.getData();
   //Nunchuck.printData();
   lcd.at(2,reads,"+");
   if (abs(Nunchuck.joyx()-127)>20||abs(Nunchuck.joyy()-127)>20 ){
     lcd.empty();
     draw(setup_str,57,1,1);//lcd.at(1,1,"Center Joystick"); 
     reads=1;
   }
   delay(25);
   
}

calibrate_joystick(Nunchuck.joyx(),Nunchuck.joyy());

 //end  Setup for Nunchuk
 lcd.empty(); 
 
// Handle EEPROM Interaction
 if( eeprom_saved() ) {
     // restore saved memory
   restore_from_eeprom_memory();
 }
 else {
     // when memory has been cleared, or nothing has been
     // saved, make sure eeprom contains default values
   write_defaults_to_eeprom_memory();
 }

//End Setup of EEPROM


//End Setup of LCD


//Setup Motors and 

init_steppers();
//setup interrupts

//SetupInputInterruptPin2HighFalling(); //use this to trigger interrupt routine to take a picture
//SetupInputInterruptPin2LowRising();
//SetupInputInterruptPin3HighFalling(); //don't use these right now
//SetupInputInterruptPin3LowRising(); //don't use pin 3 right now

}

void loop() {
  while(1) {
    switch (progstep) 
    {
           case 0:   // Move to Start Point
        Reset_or_Use_Stored();
      break;
      
      case 1:   // Move to Start Point
        Move_to_Startpoint();
      break;
            
      case 2:   // Move to End Point
        Move_to_Endpoint();
      break;
      
      case 3: //  Set Camera Interval
        Set_Cam_Interval();
      break;
     
      case 4: // 
        Set_Duration();
      break;
      
      case 5: //  
        Set_Static_Time();
      break;
         
      case 6: //  
        Set_Ramp();
      break;
       
      case 7: //  
        Set_LeadIn_LeadOut();
      break;       
        
      case 8: //  review and confirm
        Review_Confirm();
      break;
       
    case 50:  //main loop while running
    //if (DEBUG) Serial.println("entering50");
    //Serial.print("compared:");Serial.println(millis()-interval_tm);delay(10); 
    
    //Kick off the shot sequence!!!  This happens once per camera shot.   
      if( (Program_Engaged) && !(Shot_Sequence_Engaged) && ((millis() - interval_tm) > interval) || Interrupt_Fire_Engaged) { 
        interval_tm_last=interval_tm; //just used for shot timing comparison
        
        interval_tm=millis(); //start the clock on our shot sequence
        if (DEBUG) {Serial.print("trueinterval: ");Serial.print(interval_tm-interval_tm_last);Serial.print(";");}
        Interrupt_Fire_Engaged=false; //clear this flag to avoid rentering this routine

        Shot_Sequence_Engaged=true;//
        Prefire_Engaged=true;//
        IO_Engaged=true;//
        digitalWrite(FOCUS_PIN, HIGH); //for longer shot interval, wake up the camera
           //digitalWrite(IO_2, HIGH); //Use this as the iterrupt //turn these on with flags later
           //digitalWrite(IO_3, HIGH);  //Turn on Pin 3 //turn these on with flags later

          enable_steppers();;   //   LOW is motor enabled - always do this during prefire
      }
            
      //End our prefire - check that we are in program active,shot cycle engaged, and prefire engaged and check against our prefire time
      //If so set prefire flag off, static flag on, fire camera for static time value, update the display 		
     
      //if ((Program_Engaged) && (Shot_Sequence_Engaged) && (Prefire_Engaged)  && ((millis()-interval_tm) > prefire_time*100)) { 
      if ((Shot_Sequence_Engaged) && (Prefire_Engaged)  && ((millis()-interval_tm) > prefire_time*100)) { //removed requriement for Program Engaged for external interrupt
 
        Prefire_Engaged=false;
        if (DEBUG) {Serial.print("PreDoneAt ");Serial.print(millis()-interval_tm);Serial.print(";");}
        
        Static_Time_Engaged=true;
       //Fire Camera
          if (intval!=3) fire_camera((long)static_tm*100); //start shutter sequence
          camera_fired++;
          

      }

      //End out static time - check that we are in program active and static time,  Shutter not engaged,check shot cycle time agains prefire+statictime
      //If so remove flags from Static Time Engaged and IO engaged, Turn off I/O port, set flags for motors moving, move motors
      //move motors - figure out delays.   Long delays mean really slow - choose the minimum of the calculated or a good feedrate that is slow
	     
      //if (Program_Engaged && Shot_Sequence_Engaged && Static_Time_Engaged && !Shutter_Signal_Engaged && ((millis() - interval_tm) > (prefire_time*100+static_tm*100)) ) {
      if (Shot_Sequence_Engaged && Static_Time_Engaged && !Shutter_Signal_Engaged && ((millis() - interval_tm) > (prefire_time*100+static_tm*100)) ) { //removed requriement for Program Engaged for external interrupt
   
       
        Static_Time_Engaged=false; //Static Time Engaged is OFF
        IO_Engaged=false; //IO Engaged is off
          //digitalWrite(IO_2, LOW); //Use this as the iterrupt
          //digitalWrite(IO_3, LOW);  //Turn off Pin 3
          //Serial.print("IO3_off"); //Serial.println(millis()-interval_tm);
        
       
        //Move the motors - each motor move is calculated by where we are in the sequence - we still call this for lead in and lead out - motors just don't move
 
          Move_Engaged=true; //move motors  
          if (DEBUG_MOTOR) {Serial.print("MoveStart "); Serial.print(millis()-interval_tm);Serial.print(";");}
          move_motors();
          if (DEBUG_MOTOR) {Serial.print("Moveend "); Serial.print(millis()-interval_tm);Serial.print(";");}
          
        //Turn off the motors if we have selected powersave
          if (POWERSAVER)    disable_steppers();
          
        //Update display
          if (intval!=3) display_status();  //update after shot complete to avoid issues with pausing
    

       Shot_Sequence_Engaged=false;  //Shot sequence engaged flag is is off - we are ready for our next shot
       InterruptAction_Reset(); //enable the external interupts to start a new shot
       if (DEBUG) {Serial.println("EOL");} 
      }
      
      if ( camera_moving_shots > 0  && camera_fired >= camera_total_shots) { 
        lcd.empty();
        draw(setup_str,58,1,1);//lcd.at(1,1,"Program Complete"); 
        Program_Engaged=false;
        disable_steppers();
        delay(2000);
        progstep=90;
        first_time=1;
      }
    NunChuckQuerywithEC();
    NunChuckjoybuttons();
    //pause_prog(); 

    break; //break 50

case 51:  //main video loop - small and tight strip everythign that is not needed
    if (DEBUG) {
      //Serial.print("video_loop ");  
      interval_tm_last=interval_tm;
      interval_tm=millis(); 
    }
    //Start us moving  
      //if(Program_Engaged){
        if (DEBUG) {Serial.print("trueinterval ");Serial.print(interval_tm-interval_tm_last);Serial.print(";");}
      //  enable_steppers();;   //   LOW is motor enabled - always do this during prefire
      //}
            
         
        if (DEBUG_MOTOR) {Serial.print("MoveStart "); Serial.print(millis());Serial.print(";");}
        camera_fired ++; //still need this for framing
        move_motors();
        if (DEBUG_MOTOR) {Serial.print("Moveend "); Serial.println(millis());}
          

      if ( camera_total_shots > 0  && camera_fired >= camera_total_shots) { 
        lcd.empty();
        draw(setup_str,58,1,1);//lcd.at(1,1,"Program Complete"); 
        Program_Engaged=false;
        disable_steppers();
        delay(2000);
        progstep=90;
        first_time=1;
        //delay(100);
        //NunChuckQuerywithEC();
      }

break; //break 51


case 90: //  review and confirm
          if (first_time==1){
            lcd.empty();
            draw(setup_str,58,1,1);//lcd.at(1,1,"Program Complete");
            draw(setup_str,59,2,1);//lcd.at(2,1," Repeat Press C");
            NunChuckQuerywithEC();
            first_time=0;
            delay(100);
          }
          NunChuckQuerywithEC();
          NunChuckjoybuttons();
          button_actions90();  //read buttons, look for c button press to start run
break;  // break 90
    
   

    } //switch
  } // while
} //loop







